from termcolor import colored

fixed_width = 80

def output_control(func):

    def wrapper(cls, *args, **kwargs):
        if cls.output:
            func(cls, *args, **kwargs)

    return wrapper


class Output():

    color = True
    output = True
    debug = False

    @classmethod
    def _print_colored(cls, text='', color='blue', end='\n'):
        if cls.color:
            print(colored(text, color), end=end)
        else:
            print(colored(text), end=end)

    @classmethod
    def set_color_policy(cls, color):
        cls.color = color

    @classmethod
    def set_output_policy(cls, output):
        cls.output = output

    @classmethod
    def set_debug_policy(cls, debug):
        cls.debug = debug

    @classmethod
    @output_control
    def title(cls, name):
        cls.step(name, 'white', '-')

    @classmethod
    @output_control
    def start_command(cls, name):
        cls.step(name, 'white', '=', '[', ']')

    @classmethod
    @output_control
    def end_command(cls):
        cls.step('', 'white', '=', no_space=True)

    @classmethod
    @output_control
    def start_step(cls, name):
        cls.step(name, 'yellow', '-')

    @classmethod
    @output_control
    def error_step(cls, name):
        cls.step(name, 'red', '-')

    @classmethod
    def step(cls, name, text_color, character, before='', after='', no_space=False):

        str_size = len(name)

        if (str_size % 2) == 0:
            number_of_dash_left = int((fixed_width - str_size) / 2)
            number_of_dash_right = number_of_dash_left

        else:
            number_of_dash_left = int((fixed_width - str_size) / 2)
            number_of_dash_right = number_of_dash_left + 1

        number_of_dash_left -= len(before)
        number_of_dash_right -= len(after)

        space = ' '
        if no_space:
            space = ''
            number_of_dash_left += 1
            number_of_dash_right += 1

        dash_left = ''
        for i in range(0, number_of_dash_left):
            dash_left += character
        dash_left += before

        dash_right = after
        for i in range(0, number_of_dash_right):
            dash_right += character

        cls._print_colored('{0}{3}{1}{3}{2}'.format(dash_left, name, dash_right, space), text_color)

    @classmethod
    @output_control
    def category(cls, message):
        cls._print_colored('[' + message + ']', 'magenta')

    @classmethod
    @output_control
    def error(cls, message):
        cls._print_colored(message, 'red')

    @classmethod
    @output_control
    def warning(cls, message):
        cls._print_colored(message, 'orange')

    @classmethod
    @output_control
    def info(cls, message):
        cls._print_colored(message, 'cyan')

    @classmethod
    @output_control
    def infos(cls, infos, attrs):

        text = ''
        spaces = {}
        shift = 2

        for info in infos:
            for attr in attrs:
                name = attr[0]
                if name in spaces:
                    if len(info[name]) + shift > spaces[name]:
                        spaces[name] = len(info[name]) + shift
                else:
                    spaces[name] = len(info[name]) + shift

        for info in infos:
            for attr in attrs:
                name = attr[0]
                color = attr[1]
                text = ''
                if len(attr) > 2:
                    text = attr[2]

                if isinstance(info[name], tuple):
                    full_text = text + info[name][0]
                    color = info[name][1]
                else:
                    full_text = text + info[name]

                cls._print_colored(full_text, color, end='')
                cls._print_colored(' ' * (spaces[name] - len(info[name])), end='')
            cls._print_colored()



from .utensil import Utensil
from .empy import Empy
from .front_matter import FrontMatter
from .html_to_tex import HTMLToTeX
from .markdown_to_html import MarkdownToHTML
from .metadata import YamlMetadata, TomlMetadata, JsonMetadata
from .file_reader import FileReader



import os, yaml, em, importlib, tempfile, sys

from . import DocumentException
from .tools.output import Output
from .tools import read_file, write_file, DictAsMember
from .config import get_kooki_dir

class Kitchen():

    def __init__(self):

        self._config = DictAsMember()

    def cook(self, name, document):

        try:
            Output.start_step('jars')
            self._config['jars'] = check_jars(document)

            recipe_counter = 1
            recipes_len = len(document.recipe)

            for recipe_module in document.recipe:

                Output.title('{0} ({1}/{2})'.format(name, recipe_counter, recipes_len))

                Output.start_step('recipe')
                infos = []

                self._config['temp_dir'] = tempfile.mkdtemp()
                found_recipe = False

                try:
                    module = importlib.import_module(recipe_module)
                    infos.append({'name': recipe_module, 'status': '[found]'})
                    Output.infos(infos, [('name', 'blue'), ('status', 'green')])
                    found_recipe = True
                except ImportError as e:
                    infos.append({'name': recipe_module, 'status': ('[missing]', 'red')})
                    Output.infos(infos, [('name', 'blue'), ('status', 'green')])
                    raise(e)

                if found_recipe:
                    recipe = getattr(module, 'recipe')
                    recipe(document, self._config)

                if Output.debug:
                    Output.info('Temporary files: {0}'.format(self._config.temp_dir))

                recipe_counter += 1

            if not document.recipe:
                Output.title(name)
                Output.start_step('recipe')
                Output.error('no recipe provided for this target')

        except AttributeError as e:
            Output.error_step('errors')
            Output.error(e)
            sys.exit()

        except ImportError as e:
            Output.error_step('errors')
            Output.error(e)
            sys.exit()

        except DocumentException as e:
            Output.error_step('errors')
            Output.error(e)
            sys.exit()

def check_jars(document):

    infos = []
    error = False
    jars = []

    resources_dir = get_kooki_dir()
    user_jars_dir = os.path.join(resources_dir, 'jars')

    if len(document.jars) > 0:

        for package in document.jars:

            package_path = os.path.join(user_jars_dir, package)

            if os.path.isdir(package_path):
                infos.append({'name': package, 'status': '[found]', 'path': package_path})
                jars.append(package_path)
            else:
                infos.append({'name': package, 'status': ('[missing]', 'red'), 'path': ''})
                error = True

        Output.infos(infos, [('name', 'blue'), ('status', 'green'), ('path', 'cyan')])

        if error: raise DocumentException('Missing jars')

    else:
        Output.info('No jars')

    return jars

import argparse
from vcstool.commands.export import ExportCommand
from vcstool.crawler import find_repositories
from vcstool.executor import ansi, execute_jobs, generate_jobs, output_repositories, output_results


def freeze_kooki(version):
    Output.start_step('kooki')
    Output._print_colored('version ', 'blue', end='')
    Output._print_colored(version, 'cyan')
    return version


def freeze_jars(document):

    infos = []
    jars = {}

    resources_dir = get_kooki_dir()
    user_jars_dir = os.path.join(resources_dir, 'jars')

    Output.start_step('jars')

    if len(document.jars) > 0:

        for jar in document.jars:

            jar_path = os.path.join(user_jars_dir, jar)

            Output._print_colored(jar, 'blue')
            Output._print_colored('  path: ', 'yellow', end='')

            if os.path.isdir(jar_path):
                export_args = argparse.Namespace()
                export_args.path = jar_path
                export_args.exact = True
                command = ExportCommand(export_args)
                clients = find_repositories([jar_path])
                jobs = generate_jobs(clients, command)
                results = execute_jobs(jobs)

                Output._print_colored(jar_path, 'cyan')
                Output._print_colored('  vcs: ', 'yellow', end='')

                if results == []:
                    Output._print_colored('[missing]', 'red')
                elif len(results) == 1:
                    result = results[0]
                    export_data = result['export_data']
                    vcs_type = result['client'].__class__.type
                    vcs_url = export_data['url']
                    vcs_version = export_data['version']

                    Output._print_colored(vcs_type, 'cyan')
                    Output._print_colored('  url: ', 'yellow', end='')
                    Output._print_colored(vcs_url, 'cyan')
                    Output._print_colored('  version: ', 'yellow', end='')
                    Output._print_colored(vcs_version, 'cyan')

                    save_jar = {}
                    save_jar['type'] = vcs_type
                    save_jar['url'] = vcs_url
                    save_jar['version'] = vcs_version
                    jars[jar] = save_jar

            else:
                Output._print_colored('[missing]', 'red')

    else:
        Output.info('No jars')

    return jars


def check_kooki_freeze(version, freeze_data):

    # check kooki version
    Output.start_step('kooki')
    if freeze_data['kooki'] != version:
        Output._print_colored('version needed ', 'blue', end='')
        Output._print_colored(freeze_data['kooki'], 'cyan', end='')
        Output._print_colored(', version installed ', 'blue', end='')
        Output._print_colored(version, 'cyan', end='')
        Output._print_colored(' [bad kooki version]', 'red')
    else:
        Output._print_colored('version ', 'blue', end='')
        Output._print_colored(version, 'cyan', end='')
        Output._print_colored(' [good kooki version]', 'green')

    jars = get_jars()

    Output.start_step('jars')
    for jar in freeze_data['repositories']:
        version_freezed = freeze_data['repositories'][jar]['version']
        Output._print_colored(jar, 'blue')
        Output._print_colored('  freezed: ', 'yellow', end='')
        Output._print_colored(version_freezed, 'cyan')
        Output._print_colored('  installed: ', 'yellow', end='')
        if jar in jars:
            version_installed = jars[jar]['version']
            if version_freezed == version_installed:
                Output._print_colored(version_installed, 'green')
            else:
                Output._print_colored(version_installed, 'red', end='')
                Output._print_colored(' [bad version installed]', 'red')
        else:
            Output._print_colored('[jar not installed]', 'red')


from vcstool.commands.import_ import ImportCommand
from vcstool.commands.import_ import generate_jobs as import_generate_jobs
from vcstool.commands.import_ import add_dependencies as import_add_dependencies
from vcstool.commands.import_ import get_repositories as import_get_repositories


def apply_kooki_freeze(freeze_data, freeze_file):
    jars = get_jars()
    Output.start_step('jars')
    for jar in freeze_data['repositories']:
        version_freezed = freeze_data['repositories'][jar]['version']
        Output._print_colored(jar, 'blue')
        Output._print_colored('  freezed: ', 'yellow', end='')
        Output._print_colored(version_freezed, 'cyan')
        Output._print_colored('  installed: ', 'yellow', end='')
        if jar in jars:
            version_installed = jars[jar]['version']
            if version_freezed == version_installed:
                Output._print_colored(version_installed, 'green')
            else:
                Output._print_colored('Installing new version')
                apply_vcs_import(freeze_file)
        else:
            Output._print_colored('Installing new jar')
            apply_vcs_import(freeze_file)


def apply_vcs_import(freeze_file):
    resources_dir = get_kooki_dir()
    user_jars_dir = os.path.join(resources_dir, 'jars')
    repos = import_get_repositories(freeze_file)
    import_args = argparse.Namespace()
    import_args.path = user_jars_dir
    import_args.force = False

    jobs = import_generate_jobs(repos, import_args)
    import_add_dependencies(jobs)

    results = execute_jobs(jobs, show_progress=True, number_of_workers=1, debug_jobs=False)
    output_results(results)


def get_jars():
    resources_dir = get_kooki_dir()
    user_jars_dir = os.path.join(resources_dir, 'jars')

    jars = {}

    for jar in os.listdir(user_jars_dir):
        jar_path = os.path.join(user_jars_dir, jar)
        export_args = argparse.Namespace()
        export_args.path = jar_path
        export_args.exact = True
        command = ExportCommand(export_args)
        clients = find_repositories([jar_path])
        jobs = generate_jobs(clients, command)
        results = execute_jobs(jobs)

        if len(results) == 1:
            result = results[0]
            if 'export_data' in result:
                export_data = result['export_data']
                vcs_type = result['client'].__class__.type
                vcs_url = export_data['url']
                vcs_version = export_data['version']
                save_jar = {}
                save_jar['type'] = vcs_type
                save_jar['url'] = vcs_url
                save_jar['version'] = vcs_version
                jars[jar] = save_jar

    return jars



import unittest
import os

from kooki.config import get_kooki_dir

class TestKookiDir(unittest.TestCase):

    def test_env_not_set(self):
        os.environ['KOOKI_DIR'] = ''
        kooki_dir = get_kooki_dir()
        self.assertEqual(kooki_dir, os.path.expanduser('~/.kooki'))

    def test_env_set(self):
        os.environ['KOOKI_DIR'] = '/home'
        kooki_dir = get_kooki_dir()
        self.assertEqual(kooki_dir, '/home')
        os.environ['KOOKI_DIR'] = ''



from . import Step

try:
    from PyQt5.QtGui import QPageSize, QPageLayout
    from PyQt5.QtCore import QUrl, QCoreApplication, QObject, QFile, QIODevice, QEventLoop, QSizeF, Qt, QMarginsF
    from PyQt5.QtWidgets import QApplication
    from PyQt5.QtWebEngineWidgets import QWebEngineView

    class QtPDF(Step):

        def __init__(config):
            super().__init__(config)

        def __call__(self, name, content, metadata, fonts):

            filename = '{0}.html'.format(name)
            output = '{0}.pdf'.format(name)

            width = 210
            if 'width' in metadata:
                width = metadata['width']

            height = 297
            if 'height' in metadata:
                height = metadata['height']

            write_file(filename, content)

            self.process_file(filename, width, height, output)

        def process_file(self, filename, width, height, output):

            from pyvirtualdisplay import Display
            xephyr = Display(visible=False, size=(320, 240), backend='xvfb').start()

            web = HTMLPDFProcessor.Render(os.path.join(os.getcwd(), filename))

            page_size = QPageSize(QSizeF(width, height), QPageSize.Millimeter)
            layout = QPageLayout(page_size, QPageLayout.Portrait, QMarginsF())

            def get_callback(url):

                def callback(data):

                    file = QFile(url)
                    file.open(QIODevice.WriteOnly)
                    file.write(data)
                    web.app.quit()

                return callback

            web.page().runJavaScript('''
var section_height = $('section').outerHeight();

real_section_height = section_height - (2 * (section_height * 0.1))

$('.slide').each(function(index)
{
  var slide_height = $(this).outerHeight();

  if(slide_height > real_section_height)
  {
    var ratio = real_section_height / slide_height;
    $(this).css('zoom', ratio);
  }
});
''')
            web.page().printToPdf(get_callback(output), layout)

            web.app.exec_()

        class Render(QWebEngineView):

            def __init__(self, file):

                self.html = None
                self.app = QApplication(sys.argv)
                QWebEngineView.__init__(self)
                self.loadFinished.connect(self._loadFinished)

                self.load(QUrl('file://{0}'.format(file)))

                while self.html is None:
                    self.app.processEvents(QEventLoop.ExcludeUserInputEvents | QEventLoop.ExcludeSocketNotifiers | QEventLoop.WaitForMoreEvents)

            def _callable(self, data):
                self.html = data

            def _loadFinished(self, result):
                self.page().toHtml(self._callable)

except:

    class QtPDF(Step):
        pass



import os

from kooki.kooki import Kooki
from kooki.tools.output import Output
from kooki import DocumentException

from . import Step

class CssContent:

    def __init__(self):
        self.files = {}

    def __call__(self, *args):
        content = ''
        for arg in args:
            if arg in self.files:
                content += '<style>{0}</style>\n'.format(self.files[arg]())
        return content

    def add(self, name, css):
        self.files[name] = css

class Css(Step):

    def __init__(self, config):
        super().__init__(config)
        self._static = CssContent()
        self._infos = []
        self._name = 'css'
        self._config = config
        self._file_extension = '.css'

    def __call__(self):

        Output.start_step(self._name)

        for jar_path in self._config.jars:
            self._load_extensions(jar_path, self._file_extension)

        local_dir_path = os.path.join(os.getcwd(), 'kooki')
        self._load_extensions(local_dir_path, self._file_extension)

        Output.infos(self._infos, [('name', 'blue'), ('path', 'cyan')])

        return self._static

    def _load_extensions(self, directory, file_extension):

        if os.path.isdir(directory):
            self._load_extensions_rec(directory, file_extension)

    def _load_extensions_rec(self, directory, file_extension):

        for file_name in os.listdir(directory):

            if not os.path.isdir(os.path.join(directory, file_name)) and file_name.endswith(file_extension):

                extension_name = os.path.splitext(file_name)[0]
                path_to_extension = os.path.join(directory, file_name)

                kooki = Kooki(path_to_extension)
                parts = extension_name.split('.')
                self._static.add(extension_name, kooki)
                self._infos.append({'name': parts[-1], 'path': path_to_extension})

            else:

                sub_directory = os.path.join(directory, file_name)

                if os.path.isdir(sub_directory):

                    self._load_extensions_rec(sub_directory, file_extension)



import subprocess, os, shutil

from kooki.tools import write_file
from kooki.tools.output import Output
from kooki import DocumentException

from . import Step

class Xelatex(Step):

    def __init__(self, config):

        super().__init__(config)

        self._name = 'pdf'

    def __call__(self, name, document):

        Output.start_step(self._name)

        tex_file_name = '{0}.tex'.format(name)
        pdf_file_name = '{0}.pdf'.format(name)

        tex_file_path = os.path.join(self.temp_dir, tex_file_name)
        pdf_file_path = os.path.join(self.temp_dir, pdf_file_name)

        write_file(tex_file_path, document)

        command = 'xelatex -interaction=nonstopmode -halt-on-error -output-directory={1} {0}'.format(tex_file_name, self.temp_dir)
        log_file = os.path.join(self.temp_dir, 'xelatex.log')

        with open(log_file, "w") as f:
            subprocess.call(command, shell=True, stdout=f)
            subprocess.call(command, shell=True, stdout=f)

        if Output.debug:
            Output.info('XeLaTeX output: cat {0}'.format(log_file))

        if os.path.isfile(pdf_file_path):
            shutil.copy(pdf_file_path, os.getcwd())
            Output.info('PDF generated: {0}'.format(pdf_file_name))
        else:
            raise DocumentException('pdf is missing')



import yaml
from kooki.version import __version__

from kooki.config_file_parser import parse_args
from kooki.kitchen import freeze_jars, freeze_kooki, check_kooki_freeze, apply_kooki_freeze
from kooki.tools import write_file, read_file

from .command import Command

__command__ = 'freeze'
__description__ = 'Freeze version of Kooki and Jars'

class FreezeCommand(Command):

    def __init__(self):
        super(FreezeCommand, self).__init__(__command__, __description__)
        self.add_argument('documents', nargs='*')
        self.add_argument('-f', '--config-file', default='kooki.yaml')

    def callback(self, args):
        documents = parse_args(args)
        freeze_kooki(__version__)

        jars = {}
        for name, document in documents.items():
            jars.update(freeze_jars(document))

        repositories = {
            'kooki': __version__,
            'repositories': jars}
        write_file('.kooki_freeze', yaml.safe_dump(repositories, default_flow_style=False))



import yaml
import toml
import json
from kooki import DocumentException
from . import Utensil

class YamlMetadata(Utensil):

    def __call__(self, **kwargs):

        if 'content' in kwargs:
            content = kwargs['content']
        else:
            raise DocumentException('YamlMetadata: argument error in step')

        yaml_content = yaml.load(content)
        if yaml_content == None:
            yaml_content = {}
        return {'metadata': yaml_content}

class TomlMetadata(Utensil):

    def __call__(self, **kwargs):

        if 'content' in kwargs:
            content = kwargs['content']
        else:
            raise DocumentException('TomlMetadata: argument error in step')

        return {'metadata': toml.loads(content)}

class JsonMetadata(Utensil):

    def __call__(self, **kwargs):

        if 'content' in kwargs:
            content = kwargs['content']
        else:
            raise DocumentException('JsonMetadata: argument error in step')

        return {'metadata': json.loads(content)}



import os

from kooki.kooki import Kooki
from kooki.tools.output import Output
from kooki import DocumentException

from . import Step

class Static(Step):

    def __init__(self, config):

        super().__init__(config)

        self._static = {}
        self._infos = []
        self._name = 'static'
        self._config = config

    def __call__(self, filters):

        Output.start_step(self._name)

        for file_extension in filters:

            for jar_path in self._config.jars:
                self._load_extensions(jar_path, file_extension)

            local_dir_path = os.path.join(os.getcwd(), 'kooki')
            self._load_extensions(local_dir_path, file_extension)

        Output.infos(self._infos, [('name', 'blue'), ('path', 'cyan')])

        return self._static

    def _load_extensions(self, directory, file_extension):

        if os.path.isdir(directory):
            self._load_extensions_rec(directory, file_extension)

    def _load_extensions_rec(self, directory, file_extension):

        for file in os.listdir(directory):

            if file.endswith(file_extension):

                extension_name = os.path.splitext(file)[0]
                path_to_extension = os.path.join(directory, file)

                kooki = Kooki(path_to_extension)

                parts = extension_name.split('.')
                current_level = self._static

                for part in parts[:-1]:

                    if part not in current_level:
                        current_level[part] = {}

                    current_level = current_level[part]

                current_level[parts[-1]] = kooki
                self._infos.append({'name': parts[-1], 'path': path_to_extension})

            else:

                sub_directory = os.path.join(directory, file)

                if os.path.isdir(sub_directory):

                    self._load_extensions_rec(sub_directory, file_extension)



import sys

class ProgressBar():

    _bar_length = 20
    _running = False
    _progress = 0
    _message = ''
    _status = ''

    @classmethod
    def new(cls, message=''):

        if cls._running:
            cls._status = 'Abort...\r\n'
            cls._update(cls._progress)

        cls._message = message
        cls._status = ''
        cls._update(0)
        cls._running = True

    @classmethod
    def update(cls, progress):
        cls._update(progress)

    @classmethod
    def finish(cls):
        cls._update(1.0)
        cls._running = False

    @classmethod
    def _update(cls, progress):

        cls._progress = progress

        if isinstance(progress, int):
            progress = float(progress)

        if not isinstance(progress, float):
            progress = 0
            cls._status = 'error: progress var must be float\r\n'

        if progress < 0:
            progress = 0
            cls._status = 'Halt...\r\n'

        if progress >= 1:
            progress = 1
            cls._status = 'Done...\r\n'

        block = int(round(cls._bar_length * progress))
        text = "\r{2}: [{0}] {1:.2f}% {3}".format( "#" * block + "-" * (cls._bar_length - block), progress * 100, cls._message, cls._status)
        sys.stdout.write(text)
        sys.stdout.flush()



from kooki.tools import read_file
from kooki import DocumentException
from . import Utensil

class FileReader(Utensil):

    def __call__(self, **kwargs):

        arguments = kwargs

        if 'source' in arguments:
            source = arguments['source']
        else:
            raise DocumentException('FileReader: argument error in step')

        content = read_file(source)
        arguments.update({'content': content})

        return arguments



from kooki.steps import Content, Extension, Metadata, Document, Xelatex
from kooki.tools import DictAsMember

import em

def recipe(document, config):

    extension_step = Extension(config)
    extensions = extension_step(filters=['.tex'])

    metadata_step = Metadata(config)
    metadata = metadata_step(metadata=document.metadata)

    content_step = Content(config)
    content = content_step(files=document.contents, metadata={**extensions, **metadata})

    document_step = Document(config)
    document_content = document_step(template=document.template, extensions=extensions, metadata={
        **extensions,
        **metadata,
        'content': content})

    interpreter = em.Interpreter()
    interpreter.setPrefix('@')
    metadata_member = DictAsMember.convert(metadata)
    name = interpreter.expand(document.name, metadata_member)

    xelatex_step = Xelatex(config)
    xelatex_step(name=name, document=document_content)



from kooki.tools import get_extension, read_file, write_file
from kooki.document_exception import DocumentException

import yaml

class Kooki():

    def __init__(self, source):

        try:
            self._content = read_file(source)
            self._extension = get_extension(source)

            self._recipes = []

        except FileNotFoundError as e:
            raise DocumentException('No such file: {0}'.format(source))

    def __call__(self, **kwargs):
        result = self.cook(**{'metadata': kwargs})
        return result['content']

    def cook(self, **kwargs):

        result = {'content': self._content}
        result.update(kwargs)

        for recipe in self._recipes:
            recipe_result = recipe(**result)
            result.update(recipe_result)

        return result

    def content(self):
        return self._content

    def extension(self):
        return self._extension

    def add_recipe(self, recipe):
        self._recipes.append(recipe)

    def remove_ustensil(self):
        self._recipes.pop()



import time, os, yaml
import argparse

from . import Document, DocumentException
from kooki.tools.output import Output
from kooki.tools import read_file

class Config():

    def __init__(self):

        self.jars = []
        self.metadata = []
        self.content = []
        self.template = ''
        self.name = 'noname'
        self.recipe = []

    def __str__(self):
        return 'Config:\n{0}\n{1}\n{2}\n{3}\n{4}\n'.format(self.jars, self.metadata, self.content, self.template, self.name)

    def copy(self):

        config = Config()
        config.jars = self.jars.copy()
        config.metadata = self.metadata.copy()
        config.content = self.content.copy()
        config.template = self.template
        config.name = self.name
        config.recipe = self.recipe

        return config


def parse_args(args):

    documents = []

    if os.path.isfile(args.config_file):

        with open(args.config_file, 'r', encoding='utf8') as stream:

            config_raw = stream.read()
            config = yaml.load(config_raw)
            documents = parse_config(config)

            if args.documents != []:
                new_documents = {}

                for name in args.documents:
                    if name not in documents:
                        documents_availables = documents.keys()
                        documents_availables_str = '['
                        for key in list(documents_availables)[:-1]:
                            documents_availables_str += '\'' + key + '\', '
                        documents_availables_str += '\'' + list(documents_availables)[-1] + '\']'
                        raise DocumentException('No such rules for document: \'{0}\'\nRules availables {1}'.format(name, documents_availables_str))
                    else:
                        new_documents[name] = documents[name]
                documents = new_documents

    else:
        raise DocumentException('No such config file: {0}'.format(args.config_file))

    return documents


def parse_config(config_dict):

    documents = {}
    documents_config = {}

    config = Config()

    for key, value in config_dict.items():

        if key == 'template':
            config.template = value
        elif key == 'name':
            config.name = value
        elif key == 'recipe':
            parse_list('recipe', config.recipe, key, value)
        elif key == 'jars':
            parse_list('jars', config.jars, key, value)
        elif key == 'metadata':
            parse_list('metadata', config.metadata, key, value)
        elif key == 'content':
            parse_list('content', config.content, key, value)
        else:
            documents_config[key] = value
            config.name = key

    for document_key, document_args in documents_config.items():
        documents[document_key] = parse_document_config(document_args, config.copy())

    return documents


def parse_document_config(args, config):

    if isinstance(args, dict):

        for key, value in args.items():

            parse_list('jars', config.jars, key, value)
            parse_list('metadata', config.metadata, key, value)
            parse_list('content', config.content, key, value)
            parse_list('recipe', config.recipe, key, value)

            if 'template' == key: config.template = value
            if 'name' == key: config.name = value

    else:
        raise DocumentException('invalid kooki.yaml file')

    return Document(config.name, config.jars, config.template, config.content, config.metadata, config.recipe)


def parse_list(arg_key, arg_list, key, value):

    if arg_key == key and value is not None:
        if not isinstance(value, list):
            value = [value]
        arg_list += value
        return True
    else:
        return False


def parse_str(arg_key, arg, key, value):

    if arg_key == key:
        arg = value
        return True
    else:
        return False



import sass
import glob
import os
import base64
import em

from . import Step
from kooki.tools import read_file
from kooki.tools.output import Output

class FontContent:

    def __init__(self):
        self.fonts = {}

    def __call__(self, *args):

        content = ''

        if len(args) > 0:
            for arg in args:
                if arg in self.fonts:
                    content += '<style>{0}</style>\n'.format(self.fonts[arg]())

        else:
            for font in fonts:
                content += '<style>{0}</style>\n'.format(font)

        return content

    def add(self, name, font):
        self.fonts[name] = font


class Font(Step):

    font_types = ['black', 'extrabold', 'bold', 'semibold', 'regular', 'light', 'ultralight', 'hairline']
    font_weights = ['800', '700', '600', '500', '400', '300', '200', '100']

    otf_format = '''
@font-face
{
    font-family: \\font_name;
    src: url('data:font/opentype;base64, \\font_base64') format('opentype');
    font-weight: \\font_weight;
    font-style: normal;
}
'''

    def __init__(self, config):

        super().__init__(config)

        self._infos = []
        self._config = config
        self._name = 'fonts'

    def get(self):
        return Fonts(self.tex, self.html)

    def infos(self):
        return self._infos

    def __call__(self):

        Output.start_step(self._name)

        html = ''

        for jar_path in self._config.jars:

            content, files = self.load_files_in_directory(
                self.html_font_handler, jar_path, '.otf')
            html += content

            for font_file in files:
                self._infos.append({'name': font_file['path'], 'path': font_file['path']})

            content, files = self.load_files_in_directory(
                self.html_font_handler, jar_path, '.ttf')
            html += content

            for font_file in files:
                self._infos.append({'name': font_file['path'], 'path': font_file['path']})

        # Output.infos(self._infos, [('name', 'blue'), ('path', 'cyan')])

        return '<style>{0}</style>'.format(html)

    def html_font_handler(self, file):

        filename_with_ext = file.split('/')[-1]
        filename = filename_with_ext.split('.')[0]
        filename_strutured = filename.split('-')

        font_name = filename_strutured[0].lower()
        font_weight = '400'
        font_base64 = ''
        font_formatted = ''

        if len(filename_strutured) > 1:

            font_type_extracted = filename_strutured[1].lower()
            index = 0

            for font_type in self.font_types:

                if font_type == font_type_extracted:
                    font_weight = self.font_weights[index]
                    break

                if self.font_weights[index] == font_type_extracted:
                    font_weight = self.font_weights[index]
                    break

                index += 1

        with open(file, 'rb') as stream:

            content = stream.read()

            self._infos.append({'name': filename_with_ext, 'path': file})

            font_base64 = base64.b64encode(content)

            font_formatted = self.compile_font_to_otf(font_name, font_weight, str(font_base64.decode('utf8')))

        return font_formatted

    def compile_font_to_otf(self, font_name, font_weight, font_base64):

        data = {}
        data['font_name'] = font_name
        data['font_weight'] = font_weight
        data['font_base64'] = font_base64

        empy = em.Interpreter()
        empy.setPrefix('\\')
        font = empy.expand(self.otf_format, data)

        return font

    def tex_font_handler(self, file):

        filename_with_ext = file.split('/')[-1]
        filename = filename_with_ext.split('.')[0]
        font_formatted = ''

        with open(file, 'rb') as stream:

            content = stream.read()

            self._infos.append({'name': filename, 'path': ''})

            font_formatted = '\\newfontfamily\%s[]{%s}\n' % (filename.replace('-', ''), filename_with_ext)

        return font_formatted



from kooki.kooki import Kooki
from kooki.tools.output import Output
from kooki.utensils import YamlMetadata, TomlMetadata, JsonMetadata
from kooki import DocumentException

import os

from . import Step

def data_merge(a, b):
    """merges b into a and return merged result

    NOTE: tuples and arbitrary objects are not handled as it is totally ambiguous what should happen"""

    class MergeError(Exception):
        pass

    key = None

    try:
        if a is None or isinstance(a, str) or isinstance(a, bytes) or isinstance(a, int) or isinstance(a, float):
            # border case for first run or if a is a primitive
            a = b
        elif isinstance(a, list):
            # lists can be only appended
            if isinstance(b, list):
                # merge lists
                a.extend(b)
            else:
                # append to list
                a.append(b)
        elif isinstance(a, dict):
            # dicts must be merged
            if isinstance(b, dict):
                for key in b:
                    if key in a:
                        a[key] = data_merge(a[key], b[key])
                    else:
                        a[key] = b[key]
            else:
                raise MergeError('Cannot merge non-dict "%s" into dict "%s"' % (b, a))
        else:
            raise MergeError('NOT IMPLEMENTED "%s" into "%s"' % (b, a))
    except TypeError as e:
        raise MergeError('TypeError "%s" in key "%s" when merging "%s" into "%s"' % (e, key, b, a))
    return a

class Metadata(Step):

    def __init__(self, config):

        super().__init__(config)

        self._infos = []
        self._name = 'metadata'
        self._config = config
        self.global_metadata = {}

    def __call__(self, metadata):

        metadata_dict = {}
        missing = False

        Output.start_step(self._name)

        for jar_path in self._config.jars:
            self._load_from_jars(jar_path, '.yaml')
            self._load_from_jars(jar_path, '.yml')
            self._load_from_jars(jar_path, '.toml')
            self._load_from_jars(jar_path, '.json')

        for source in metadata:

            try:
                if os.path.isfile(source):
                    path_to_source = os.path.join(os.getcwd(), source)
                elif source in self.global_metadata:
                    path_to_source = self.global_metadata[source]
                else:
                    raise Exception()

                kooki = self._load(path_to_source)

                kooki_cooked = kooki.cook()
                metadata_dict = data_merge(metadata_dict, kooki_cooked['metadata'])
                self._infos.append({'name': source, 'status': '[found]', 'path': path_to_source})
            except OSError:
                self._infos.append({'name': source, 'status': ('[missing]', 'red'), 'path': ''})
                missing = True

        if len(metadata) > 0:
            Output.infos(self._infos, [('name', 'cyan'), ('status', 'green'), ('path', 'cyan')])
        else:
            Output.info('no metadata')

        if missing:
            raise DocumentException('Missing files or sources')

        return metadata_dict

    def _load(self, source):

        kooki = Kooki(source)

        extension = kooki.extension()
        recipe = None

        if extension == '':
            recipe = self._guess_recipe(kooki.content())

        elif extension == 'yaml' or extension == 'yml':
            recipe = YamlMetadata()

        elif extension == 'toml':
            recipe = TomlMetadata()

        elif extension == 'json':
            recipe = JsonMetadata()

        if recipe is not None:
            kooki.add_recipe(recipe)

        return kooki

    def _load_from_jars(self, jars, filters):

        for package_dir_path in self._config.jars:
            self._load_metadata(package_dir_path, filters)

    def _load_metadata(self, directory, file_extension):

        if os.path.isdir(directory):
            self._load_metadata_rec(directory, file_extension)

    def _load_metadata_rec(self, directory, file_extension):

        for file in os.listdir(directory):

            if file.endswith(file_extension):

                extension_name = os.path.splitext(file)[0]
                path_to_extension = os.path.join(directory, file)
                self.global_metadata[file] = path_to_extension

            else:

                sub_directory = os.path.join(directory, file)

                if os.path.isdir(sub_directory):

                    self._load_metadata_rec(sub_directory, file_extension)

    def _guess_recipe(self, content):

        processors = []
        processors.append(YamlMetadata)
        processors.append(TomlMetadata)
        processors.append(JsonMetadata)

        succeed = False
        recipe = None

        while len(processors) > 0 and not succeed:
            try:
                Processor = processors.pop()
                recipe = Processor()
                succeed = True
            except:
                succeed = False

        return recipe



from kooki.kooki import Kooki
from kooki.tools.output import Output
from kooki import DocumentException

from kooki.utensils import MarkdownToHTML, HTMLToTeX, Empy, FrontMatter

from . import Step

class Content(Step):

    def __init__(self, config, tex=True):

        super().__init__(config)

        self._infos = []
        self._name = 'contents'
        self._tex = tex

    def __call__(self, files, metadata):

        content = ''
        missing = False

        Output.start_step(self._name)

        for source in files:
            try:
                kooki = self._load(source, metadata)
                kooki_cooked = kooki.cook()
                content += kooki_cooked['content']
                self._infos.append({'name': source, 'status': '[found]'})
            except DocumentException as e:
                raise DocumentException('{0}\n{1}'.format(source, e))
            except Exception as e:
                self._infos.append({'name': source, 'status': ('[missing]', 'red')})
                missing = True

        if len(files) > 0:
            Output.infos(self._infos, [('name', 'cyan'), ('status', 'green')])
        else:
            Output.info('no content')

        if missing:
            raise DocumentException('Missing files ou sources')

        return content

    def _load(self, content_source, metadata):

        front_matter = FrontMatter()
        template = Empy(metadata)
        html = MarkdownToHTML()
        tex = HTMLToTeX()

        kooki = Kooki(content_source)

        kooki.add_recipe(front_matter)
        kooki.add_recipe(template)
        kooki.add_recipe(html)
        if self._tex:
            kooki.add_recipe(tex)

        return kooki



import os
import kooki

resource_dir_env = 'KOOKI_DIR'
resource_dir_default = '~/.kooki'

def get_kooki_dir():
    resource_dir = os.environ.get(resource_dir_env)

    if not resource_dir:
        resource_dir = os.path.expanduser(resource_dir_default)

    kooki_package_dir = kooki.__path__[0]
    resource_dir = os.path.join(os.path.dirname(kooki_package_dir))

    return resource_dir



import yaml

from kooki.kitchen import apply_kooki_freeze
from kooki.tools import read_file

from .command import Command

__command__ = 'update'
__description__ = 'Update a kooki project'

class UpdateCommand(Command):

    def __init__(self):
        super(UpdateCommand, self).__init__(__command__, __description__)

    def callback(self, args):
        freeze_file = read_file('.kooki_freeze')
        freeze_data = yaml.safe_load(freeze_file)
        apply_kooki_freeze(freeze_data, freeze_file)



import os

class Step():

    def __init__(self, config):
        self.temp_dir = config.temp_dir

    def load_files_in_directory(self, handler, directory, extension):

        content = ''
        files = []

        for file in sorted(os.listdir(directory)):

            if file.endswith(extension):

                extension_name = os.path.splitext(file)[0]

                content += handler(os.path.join(directory, file))
                files.append({'path': os.path.join(directory, file)})

            else:

                sub_directory = os.path.join(directory, file)

                if os.path.isdir(sub_directory):

                    sub_content, sub_files = self.load_files_in_directory(handler, sub_directory, extension)
                    content += sub_content
                    files += sub_files

        return content, files



import markdown
from kooki import DocumentException
from . import Utensil

class MarkdownToHTML(Utensil):

    def __init__(self):

        extensions = [
            'markdown.extensions.meta',
            'markdown.extensions.tables',
            'markdown.extensions.attr_list',
            'markdown.extensions.fenced_code',
            'markdown.extensions.def_list',
            'markdown.extensions.sane_lists',
            'markdown.extensions.codehilite']

        self._md = markdown.Markdown(extensions=extensions)

    def __call__(self, **kwargs):

        if 'content' in kwargs:
            content = kwargs['content']
        else:
            raise DocumentException('MarkdownToHTML: argument error in step')

        md_content = self._md.convert(content)
        return {'content': md_content}



import yaml
from kooki.version import __version__

from kooki.kitchen import check_kooki_freeze
from kooki.tools import read_file

from .command import Command

__command__ = 'check'
__description__ = 'Check version of kooki freeze'

class CheckCommand(Command):

    def __init__(self):
        super(CheckCommand, self).__init__(__command__, __description__)

    def callback(self, args):
        freeze_file = read_file('.kooki_freeze')
        freeze_data = yaml.safe_load(freeze_file)
        check_kooki_freeze(__version__, freeze_data)



from kooki import DocumentException
from kooki.tools.output import Output

from . import Step

class Document(Step):

    def __init__(self, config):

        super().__init__(config)

        self._infos = []
        self._name = 'document'

    def __call__(self, template, extensions, metadata):

        Output.start_step(self._name)
        Output.info(template)

        if template in extensions:
            kooki = extensions[template]
        else:
            raise DocumentException('bad template')

        document = kooki(**metadata)

        return document



import em
from kooki.tools import DictAsMember
from kooki import DocumentException
from . import Utensil

class Empy(Utensil):

    def __init__(self, metadata={}):

        self._interpreter = em.Interpreter()
        self._interpreter.setPrefix('@')

        self._metadata = metadata

    def __call__(self, **kwargs):

        if 'content' in kwargs:
            content = kwargs['content']
        else:
            raise DocumentException('Empy: argument error in step')

        if 'metadata' in kwargs:
            metadata = kwargs['metadata']
        else:
            metadata = {}

        metadata_copy = self._metadata.copy()
        metadata_copy.update(metadata)
        metadata_member = DictAsMember.convert(metadata_copy)

        try:
            content = self._interpreter.expand(content, metadata_member)
        except KeyError as e:
            raise DocumentException('Missing variable: {0}'.format(e))

        return {'content': content}



import sass, os, shutil

from . import Step
from kooki.kooki import Kooki
from kooki.tools import read_file
from kooki.tools.output import Output

class SassContent:

    def __init__(self):
        self.files = {}

    def add(self, name, path):
        self.files[name] = path

    def __call__(self, name):

        processed_sass = ''
        current = os.getcwd()
        path = self.files[name]
        os.chdir(path)

        with open(os.path.join(path, name + '.scss')) as f:
            processed_sass = sass.compile(string=bytes(f.read(), 'utf8'))

        os.chdir(current)
        return '<style>{0}</style>'.format(processed_sass)

class Sass(Step):

    def __init__(self, config):

        super().__init__(config)

        self._sass = SassContent()
        self._infos = []
        self._name = 'sass'
        self._config = config

    def __call__(self):

        os.mkdir(os.path.join(self.temp_dir, 'sass'))

        def handler(path):

            sass_name = os.path.splitext(path)[0].split('/')[-1]
            shutil.copy(path, os.path.join(self.temp_dir, 'sass', sass_name + '.scss'))
            self._infos.append({'name': sass_name, 'path': path})
            self._sass.add(sass_name, os.path.join(self.temp_dir, 'sass'))
            return ''

        Output.start_step(self._name)

        for jar_path in self._config.jars:
            self.load_files_in_directory(handler, jar_path, '.scss')

        Output.infos(self._infos, [('name', 'blue'), ('path', 'cyan')])

        if Output.debug:
            pass
            # log_file = os.path.join(self.temp_dir, 'sass.css')
            # with open(log_file, 'w') as f:
            #     f.write(self._sass)
            # Output.info('Sass output: cat {0}'.format(log_file))

        return self._sass



import unittest, tempfile, os, argparse

from kooki.commands.new import NewCommand
from kooki.commands.bake import BakeCommand

from kooki.tools.output import Output

Output.set_output_policy(False)

new = NewCommand()
bake = BakeCommand()

class TestSimpleNewAndBake(unittest.TestCase):

    @classmethod
    def setUpClass(cls):

        cls.new_args = argparse.Namespace()
        cls.new_args.name = ''
        cls.new_args.content = []
        cls.new_args.template = ''
        cls.new_args.recipe = ''
        cls.new_args.jars = []
        cls.new_args.metadata = []
        cls.new_args.content = []

        cls.bake_args = argparse.Namespace()
        cls.bake_args.config_file = 'kooki.yaml'
        cls.bake_args.documents = []

    def test_new_command(self):

        tempdir = tempfile.mkdtemp()
        os.chdir(tempdir)

        new.callback(TestSimpleNewAndBake.new_args)

        listdir = os.listdir()
        self.assertIn('metadata.yaml', listdir)
        self.assertIn('kooki.yaml', listdir)
        self.assertIn('content.md', listdir)

        bake.callback(TestSimpleNewAndBake.bake_args)
        listdir = os.listdir()



import os

class Document():

    def __init__(self, name, jars, template, contents, metadata, recipe):

        self.name = name
        self.jars = jars
        self.template = template
        self.contents = contents
        self.metadata = metadata
        self.recipe = recipe

        self.path = os.path.join(os.getcwd(), self.name)



import argparse, sys, platform

from kooki.version import __version__
from kooki.config_file_parser import parse_args
from kooki import Kitchen
from kooki.tools.output import Output

from .command import Command
from .bake import BakeCommand
from .freeze import FreezeCommand
from .update import UpdateCommand
from .check import CheckCommand
from .jar import JarCommand
from .new import NewCommand

__command__ = 'kooki'
__description__ = 'Generate easily any kind of documents.'

class KookiCommand(Command):

    def __init__(self):
        super(KookiCommand, self).__init__(__command__, __description__)
        self.add_argument('-v', '--version', help='Show program\'s version number and exit.', action='store_true')
        self.add_command(BakeCommand())
        self.add_command(FreezeCommand())
        self.add_command(CheckCommand())
        self.add_command(UpdateCommand())
        self.add_command(JarCommand())
        self.add_command(NewCommand())

    def callback(self, args):

        if args.version:
            Output._print_colored('Kooki', 'blue', end=' ')
            Output._print_colored(__version__, 'cyan')

            python_info = sys.version_info
            python_build = platform.python_build()
            Output._print_colored('Python', 'blue', end=' ')
            Output._print_colored('{0}.{1}.{2}'.format(python_info.major, python_info.minor, python_info.micro), 'cyan', end=' ')
            Output._print_colored('({0}, {1})'.format(python_build[0], python_build[1]), 'yellow')

            python_compiler = platform.python_compiler().split(' ')

            Output._print_colored(python_compiler[0], 'blue', end=' ')
            Output._print_colored(python_compiler[1], 'cyan', end=' ')
            Output._print_colored('({0})'.format(python_compiler[2]), 'yellow')
        else:
            print(self.parser.print_help())



#!/usr/bin/env python
import os
from setuptools import setup, find_packages

__version__ = '0.4.1'

datadir = os.path.join('jars')
data_files = [(os.path.join('/usr/share/kooki', d), [os.path.join(d,f) for f in files])
    for d, folders, files in os.walk(datadir)]

setup(
    name='kooki',
    version=__version__,
    description='The ultimate document generator.',
    author='Noel Martignoni',
    author_email='noel@martignoni.fr',
    url='https://gitlab.com/kooki/kooki',
    scripts=['scripts/kooki'],
    install_requires=['markdown', 'empy', 'pyyaml', 'toml', 'requests', 'termcolor', 'vcstool', 'libsass'],
    packages=find_packages(exclude=['tests*', 'jars']),
    test_suite='tests',
    data_files=data_files,
)



import argparse, os, sys

from kooki.tools.output import Output
from kooki.kitchen import get_jars
from .command import Command

__command__ = 'jar'
__description__ = 'List jars'

class JarCommand(Command):

    def __init__(self):
        super(JarCommand, self).__init__(__command__, __description__)

    def callback(self, args):

        jars = get_jars()

        Output.start_step('jars')
        for jar in jars:
            Output._print_colored(jar, 'blue')
            Output._print_colored('  version: ', 'yellow', end='')
            Output._print_colored(jars[jar]['version'], 'green')



from kooki.steps import Extension, Metadata, Document, Static, Sass, Font, Js, Css, Slides
from kooki.tools import write_file, DictAsMember

import em

def recipe(document, config):

    extension_step = Extension(config)
    extensions = extension_step(filters=['.html'])

    metadata_step = Metadata(config)
    metadata = metadata_step(metadata=document.metadata)

    font_step = Font(config)
    fonts = font_step()

    js_step = Js(config)
    js = js_step()

    css_step = Css(config)
    css = css_step()

    sass_step = Sass(config)
    sass = sass_step()

    slides_step = Slides(config)
    content = slides_step(files=document.contents, metadata={**extensions, **metadata})

    document_step = Document(config)
    document_content = document_step(template=document.template, extensions=extensions, metadata={
        **extensions,
        **metadata,
        'js': js,
        'css': css,
        'sass': sass,
        'fonts': fonts,
        'content': content})

    interpreter = em.Interpreter()
    interpreter.setPrefix('@')
    metadata_member = DictAsMember.convert(metadata)
    name = interpreter.expand(document.name, metadata_member)
    write_file(name + '.html', document_content)



from .step import Step
from .content import Content
from .static import Static
from .extension import Extension
from .metadata import Metadata
from .document import Document
from .css import Css
from .font import Font
from .js import Js
from .qt_pdf import QtPDF
from .sass import Sass
from .slides import Slides
from .xelatex import Xelatex



from kooki.kooki import Kooki
from kooki.tools.output import Output
from kooki import DocumentException
from kooki.tools import read_file
from kooki.utensils import MarkdownToHTML, HTMLToTeX, Empy, FrontMatter
from xml.dom import minidom

from . import Step

class ContentKooki(Kooki):

    def __init__(self, content):

        self._content = content
        self._recipes = []

class Slides(Step):

    def __init__(self, config, tex=True):
        self._infos = []
        self._name = 'contents'
        self._tex = tex

    def __call__(self, files, metadata):

        content = ''
        missing = False

        Output.start_step(self._name)

        slide_number = 1

        for source in files:
            try:
                kookis = self._load(source, metadata)

                for kooki in kookis:
                    kooki_cooked = kooki.cook()

                    classes = ''
                    html_id = ''
                    others = ''
                    slide_content = kooki_cooked['content']

                    if kooki_cooked['content'].startswith('<p>{:'):
                        end = kooki_cooked['content'].find('}</p>')
                        attrs = kooki_cooked['content'][5:end].split()
                        slide_content = kooki_cooked['content'][end + 5:]

                        for attr in attrs:

                            if attr.startswith('.'):
                                classes += attr[1:]

                            elif attr.startswith('#'):
                                html_id = attr[1:]

                            else:
                                others += attr

                    content += '<section id="{0}" class="slide slide-{3} {1}" {2}>'.format(html_id, classes, others, slide_number)
                    content += '<div class="slide-number">{0}</div>'.format(slide_number)
                    content +=  '<div class="slide-content"><div class="slide-content-child">'
                    content += slide_content
                    content += '</div></div></section>'

                    slide_number += 1

                self._infos.append({'name': source, 'status': '[found]'})

            except Exception as e:
                print(e)
                self._infos.append({'name': source, 'status': ('[missing]', 'red')})
                missing = True

        if len(files) > 0:
            Output.infos(self._infos, [('name', 'cyan'), ('status', 'green')])
        else:
            Output.info('no content')

        if missing:
            raise DocumentException('Missing files ou sources')

        return content

    def _load(self, content_source, metadata):

        template = Empy(metadata)
        html = MarkdownToHTML()

        content = read_file(content_source)

        splitted = content.split('---\n')

        kookis = []

        for part in splitted:

            kooki = ContentKooki(part)
            kooki.add_recipe(template)
            kooki.add_recipe(html)

            kookis.append(kooki)

        return kookis

    def _get_image(image_path):
        image_file = urllib.urlopen(image_path)
        image_result = ''

        if image_file.code == 200:
            image_result = _convert_image(image_file)
        else:
            image_result = _get_local_image(image_path)

        image_db[str(image_path)] = image_result

        return image_result

    def _get_local_image(image_path):
        with open(image_path, "rb") as image_file:
            return _convert_image(image_file)

    def _convert_image(image_file):
        image_content_base64 = base64.b64encode(image_file.read())
        return 'data:image/png;base64,' + image_content_base64



from kooki.tools import get_front_matter
from kooki import DocumentException
from . import Utensil
import yaml

class FrontMatter(Utensil):

    def __call__(self, **kwargs):

        arguments = kwargs

        if 'content' in arguments:
            content = arguments['content']
        else:
            raise DocumentException('Frontmatter argument error')

        if 'metadata' in arguments:
            metadata = arguments['metadata']
        else:
            metadata = {}

        front_matter, content = get_front_matter(content)

        if not isinstance(front_matter, dict):
            raise DocumentException('Bad front matter, kooki only support yaml dict front matter for now')

        front_matter.update(metadata)
        metadata = front_matter

        arguments.update({'metadata': metadata, 'content': content})

        return arguments



from markdown.inlinepatterns import LinkPattern, handleAttributes
from markdown.extensions import Extension
from markdown import util
import base64
import urllib

NOBRACKET = r'[^\]\[]*'
BRK = (
    r'\[(' +
    (NOBRACKET + r'(\[')*6 +
    (NOBRACKET + r'\])*')*6 +
    NOBRACKET + r')\]'
)
IMAGE_LINK_RE = r'\!' + BRK + r'\s*\((<.*?>|([^")]+"[^"]*"|[^\)]*))\)'

class ImagePatternBase64(LinkPattern):

    def handleMatch(self, m):

        el = util.etree.Element("img")
        src_parts = m.group(9).split()
        if src_parts:
            src = src_parts[0]
            if src[0] == "<" and src[-1] == ">":
                src = src[1:-1]

            image = get_image(self.sanitize_url(self.unescape(src)))
            el.set('src', image)
        else:
            el.set('src', "")
        if len(src_parts) > 1:
            el.set('title', dequote(self.unescape(" ".join(src_parts[1:]))))

        if self.markdown.enable_attributes:
            truealt = handleAttributes(m.group(2), el)
        else:
            truealt = m.group(2)

        el.set('alt', self.unescape(truealt))
        return el


class ImageDownloaderExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.inlinePatterns.add('image_base64', ImagePatternBase64(IMAGE_LINK_RE, md), '<image_link')


def get_image(image_path):

    try:
        image_file = urllib.request.urlopen(image_path)
        image_result = ''

        image_result = convert_image(image_file)

    except:
        image_result = get_local_image(image_path)

    return image_result

def get_local_image(image_path):

    with open(image_path, "rb") as image_file:
        return convert_image(image_file)

def convert_image(image_file):

    image_content_base64 = base64.b64encode(image_file.read())

    return 'data:image/png;base64,' + image_content_base64.decode('utf8')



import argparse

from kooki.config_file_parser import parse_args
from kooki import Kitchen

from .command import Command

__command__ = 'bake'
__description__ = 'Bake a kooki'

class BakeCommand(Command):

    def __init__(self):
        super(BakeCommand, self).__init__(__command__, __description__)
        self.add_argument('documents', nargs='*')
        self.add_argument('-f', '--config-file', default='kooki.yaml')
        self.add_argument('-h', '--help', action='help', default=argparse.SUPPRESS, help='Show this help message and exit.')
        self.add_argument('-v', '--verbose', help='Show more information about the bake processing.', action='store_true')

    def callback(self, args):
        kitchen = Kitchen()
        documents = parse_args(args)

        for name, document in documents.items():
            kitchen.cook(name, document)



from codecs import open

import requests
import em
import yaml
import os

def get_extension(filename):

    tmp = filename.split('.')

    if len(tmp) > 0:
        return tmp[-1]
    else:
        raise Exception('No extension')

def get_front_matter(content):

    first_line = content.split('\n', 1)[0]
    content_splitted = content.split('---\n')
    front_matter = {}

    if first_line == '---':

        if len(content_splitted) >= 3:
            metadata_content = content_splitted[1]
            content = '---\n'.join(content_splitted[2:])

            front_matter = yaml.load(metadata_content)
        else:
            content = '---\n'.join(content_splitted)

    return front_matter, content

def read_source(source):

    try:
        request = requests.get(source)
        content = request.text
        extension = ''

    except:
        content = read_file(source)

        split_source = source.split('.')
        extension = split_source[-1]

    return content, extension

def read_file(filename):

    content = ''

    with open(filename, 'r', encoding='utf8') as stream:
        content = stream.read()

    return content

def read_byte_file(filename):

    content = ''

    with open(filename, 'rb') as stream:
        content = stream.read()

    return content

def write_file(filename, content):

    elements = filename.split('/')
    path = ''
    for element in elements[:-1]:
        path = os.path.join(path, element)
        if path != '' and not os.path.isdir(path):
            os.mkdir(path)

    with open(filename, 'w', encoding='utf8') as stream:
        stream.write(content)

def deep_utf8(data):

    if isinstance(data, str):
        return data

    elif isinstance(data, list):
        new_data = []
        for value in data:
            new_data.append(deep_utf8(value))
        return new_data

    elif isinstance(data, dict):
        new_data = {}
        for key, value in data.items():
            new_data[key] = deep_utf8(value)
        return new_data

    else:
        return data


class DictAsMember(dict):

    @classmethod
    def convert(cls, data):

        if isinstance(data, dict):
            value = DictAsMember()
            for key in data:
                value[key] = DictAsMember.convert(data[key])
            return value
        else:
            return data

    def __getattr__(self, name):

        value = self[name]

        try:
            if isinstance(value, dict):
                value = DictAsMember(value)
            elif isinstance(value, unicode):
                data[key] = value.encode('utf8')
        except NameError:
            pass

        return value


class SafeEmpyInterpreter(em.Interpreter):

    def expand(self, content, data):

        result = ''
        missing = {}

        prefix = self.getPrefix()

        error = True
        new_data = DictAsMember(data)

        while error:
            try:
                result = super().expand(content, new_data)
                error = False
            except NameError as e:
                tag = str(e).split('\'')[1]
                new_data[tag] = DictAsMember()
                missing[tag] = ''
            except AttributeError as e:
                raise Exception('AttributeError: {0}'.format(str(e)))
            except TypeError as e:
                raise Exception('TypeError: {0}'.format(str(e)))
            except KeyError as e:
                raise Exception('KeyError: {0}'.format(str(e)))

        return result, missing


def format_data(data):

    for key, value in list(data.items()):

        try:
            if isinstance(value, dict):
                data[key] = DictAsMember(value)
            elif isinstance(value, unicode):
                data[key] = value.encode('utf8')
        except NameError:
            pass
